@@ -117,6 +117,7 @@ Examples
 """
 
 import sys
+import warnings
 import numpy as np
 from numpy import linalg as LA
 from numpy import fft as FFT
@@ -127,6 +128,7 @@ from scipy.misc import comb as SPcomb
 from scipy import sparse as scsp
 
 import SparseGrids as SG
+import orthpol
 
 JACOBI = 'Jacobi'
 HERMITEP = 'HermiteP'
@@ -256,7 +258,9 @@ class Poly1D(object):
             
             J = np.zeros((N+1,N+1))
             h1 = 2.* np.arange(0.,N+1)+alpha+beta
-            J = np.diag(-1./2.*(alpha**2.-beta**2.)/(h1+2.)/h1) +  np.diag( 2./(h1[0:N] + 2.) * np.sqrt( np.arange(1.,N+1) * (np.arange(1.,N+1)+alpha+beta) * (np.arange(1.,N+1)+alpha) * (np.arange(1.,N+1)+beta) / (h1[0:N] + 1.) / (h1[0:N] + 3.) ) , 1) 
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore")
+                J = np.diag(-1./2.*(alpha**2.-beta**2.)/(h1+2.)/h1) +  np.diag( 2./(h1[0:N] + 2.) * np.sqrt( np.arange(1.,N+1) * (np.arange(1.,N+1)+alpha+beta) * (np.arange(1.,N+1)+alpha) * (np.arange(1.,N+1)+beta) / (h1[0:N] + 1.) / (h1[0:N] + 3.) ) , 1) 
             if (alpha + beta < 10.*np.finfo(np.float64).eps): J[0,0] = 0.0
             J = J + np.transpose(J)
             
@@ -447,7 +451,7 @@ class Poly1D(object):
             x,vec = np.linalg.eig(D)
             x = np.sort(x)
             hp = self.__HermiteP_Prob(x,N)
-            w = factorial(N) / ((N+1) * hp[:,0]**2.)
+            w = factorial(N) / ((N+1) * hp**2.)
             return (x,w)
     
     def __LaguerrePGQ(self,N,alpha=None):
@@ -619,7 +623,6 @@ class Poly1D(object):
         
         .. note:: For further details see [4]_
         """
-        import orthpol
 
         eps = orthpol.d1mach(3)
         
@@ -664,7 +667,6 @@ class Poly1D(object):
         
         .. note:: For further details see [4]_
         """
-        import orthpol
         
         eps = orthpol.d1mach(3)
         
@@ -709,7 +711,6 @@ class Poly1D(object):
         
         .. note:: For further details see [4]_
         """
-        import orthpol
         
         eps = orthpol.d1mach(3)
         
@@ -899,15 +900,9 @@ class Poly1D(object):
         q = L[N+1] - L[N-1]
         dq = DL[N+1] - DL[N-1]
         return (q,dq,L[N])
-    
+
     def __JacobiP(self,x,N,alpha=None,beta=None):
-        """
-        Evaluate Jacobi Polynomial of type (alpha,beta) > -1
-        (alpha+beta <> -1) at points x for order N and returns P[1:length(xp))]
-        If alpha=-0.5 and beta=-0.5, then this function refers to the generation
-        of Chebyshev Polynomials (routine ChebyshevP)
-        Note: normalized to be orthonormal
-        """
+
         if (self.poly != JACOBI):
             print >> self.sdout, "The method __JacobiP cannot be called with the actual type of polynomials. Actual type: '%s'" % self.poly
         else:
@@ -917,33 +912,9 @@ class Poly1D(object):
             if ( AlmostEqual(alpha,-0.5) and AlmostEqual(beta,-0.5) ):
                 return self.__ChebyshevP(x,N)
             else:
-                xp = np.mat(x);
-                dims = np.shape(xp)
-                if (dims[1] == 1): xp = np.transpose(xp)
-                
-                PL = np.mat(np.zeros((N+1,np.shape(xp)[1])))
-                
-        
-                gamma0 = 2**(alpha+beta+1)/(alpha+beta+1)*gammaF(alpha+1)*gammaF(beta+1)/gammaF(alpha+beta+1)
-                
-                PL[0,:] = 1.0/np.sqrt(gamma0)
-                if (N == 0): return np.transpose(PL)
-                
-                gamma1 = (alpha+1.)*(beta+1.)/(alpha+beta+3.)*gamma0
-                PL[1,:] = ((alpha+beta+2.)*xp/2. + (alpha-beta)/2.)/np.sqrt(gamma1)
-                if (N == 1): return np.transpose(PL[N,:])
-                
-                # Recurrence
-                aold = 2./(2.+alpha+beta)*np.sqrt((alpha+1.)*(beta+1.)/(alpha+beta+3.))
-                
-                for i in range(1,N):
-                    h1 = 2.*i+alpha+beta
-                    anew = 2./(h1+2.)*np.sqrt( (i+1.)*(i+1.+alpha+beta)*(i+1.+alpha)*(i+1.+beta)/(h1+1.)/(h1+3.) )
-                    bnew = - (alpha**2. - beta**2.)/h1/(h1+2.)
-                    PL[i+1,:] = 1.0 / anew*( -aold*PL[i-1,:] + np.multiply((xp-bnew),PL[i,:]) )
-                    aold = anew
-                    
-                return np.transpose(PL[N,:])
+                (al,be,ierr) = orthpol.drecur(N+1,6,alpha,beta)
+                v = orthpol.polyeval(x,N,al,be)
+                return v
             
     def __LegendreP(self,r, N):
         """
@@ -974,7 +945,6 @@ class Poly1D(object):
             alpha,beta = self.params
             if ( AlmostEqual(alpha,-0.5) and AlmostEqual(beta,-0.5) ):
                 # shape r
-                r = np.reshape(np.array(r),(r.shape[0],1))
                 Ks = 50
                 rShape = r.shape
                 if ( N == 0 ): return np.ones(rShape)
@@ -997,18 +967,17 @@ class Poly1D(object):
         if (self.poly != HERMITEP):
             print >> self.sdout, "The method __HermiteP cannot be called with the actual type of polynomials. Actual type: '%s'" % self.poly
         else:
-            old2 = np.ones( (len(r),1) )
+            old2 = np.ones( len(r) )
             if (N == 0):
                 return old2
-            rs = np.reshape(r,(len(r),1))
-            old1 = 2. * rs
+            old1 = 2. * r
             if (N == 1):
                 return old1
-            new = 2. * rs * old1 - 2. * old2
+            new = 2. * r * old1 - 2. * old2
             for i in xrange(3,N+1):
                 old2 = old1
                 old1 = new
-                new = 2. * rs * old1 - 2. * (i-1) * old2
+                new = 2. * r * old1 - 2. * (i-1) * old2
             return new
     
     def __HermitePnorm(self,r, N):
@@ -1019,18 +988,17 @@ class Poly1D(object):
         if (self.poly != HERMITEP):
             print >> self.sdout, "The method __HermitePnorm cannot be called with the actual type of polynomials. Actual type: '%s'" % self.poly
         else:
-            old2 = np.zeros( (len(r),1) )
+            old2 = np.zeros( len(r) )
             if (N == -1):
                 return old2
-            old1 = np.ones( (len(r),1) ) / (np.pi**(1./4.))
+            old1 = np.ones( len(r) ) / (np.pi**(1./4.))
             if (N == 0):
                 return old1
-            rs = np.reshape(r,(len(r),1))
-            new = np.sqrt(2) * rs * old1
+            new = np.sqrt(2) * r * old1
             for i in xrange(2,N+1):
                 old2 = old1
                 old1 = new
-                new = np.sqrt(2./i) * rs * old1 - np.sqrt((i-1.)/i) * old2
+                new = np.sqrt(2./i) * r * old1 - np.sqrt((i-1.)/i) * old2
             return new
         
     def __HermiteF(self,r, N):
@@ -1041,7 +1009,6 @@ class Poly1D(object):
         if (self.poly != HERMITEF):
             print >> self.sdout, "The method __HermiteF cannot be called with the actual type of polynomials. Actual type: '%s'" % self.poly
         else:
-            r = np.reshape(r,(len(r),1))
             old2 = np.exp(-r**2. / 2.)
             if (N == 0):
                 return old2
@@ -1063,18 +1030,17 @@ class Poly1D(object):
         if (self.poly != HERMITEP_PROB):
             print >> self.sdout, "The method __HermiteP_Prob cannot be called with the actual type of polynomials. Actual type: '%s'" % self.poly
         else:
-            old2 = np.ones( (len(r),1) )
+            old2 = np.ones( len(r) )
             if (N == 0):
                 return old2
-            rs = np.reshape(r,(len(r),1))
-            old1 = rs
+            old1 = r.copy()
             if (N == 1):
                 return old1
-            new = rs * old1 - old2
+            new = r * old1 - old2
             for i in xrange(3,N+1):
                 old2 = old1
                 old1 = new
-                new = rs * old1 - (i-1) * old2
+                new = r * old1 - (i-1) * old2
             return new
         
     def __LaguerreP(self,r,N,alpha=None):
@@ -1138,19 +1104,18 @@ class Poly1D(object):
             if (alpha == None):
                 (alpha,) = self.params
             # Recurrence relation
-            rs = np.reshape(r,(len(r),1))
-            old2 = np.exp(-rs/2.)
+            old2 = np.exp(-r/2.)
             if (N == 0):
                 return old2
-            old1 = (alpha + 1. - rs) * np.exp(-rs/2.)
+            old1 = (alpha + 1. - r) * np.exp(-r/2.)
             if (N == 1):
                 return old1
-            new = ( (3. + alpha - rs) * old1 - (1. + alpha) * old2 ) / 2.0
+            new = ( (3. + alpha - r) * old1 - (1. + alpha) * old2 ) / 2.0
             for i in xrange(3,N+1):
                 n = i - 1.
                 old2 = old1
                 old1 = new
-                new = ( (2*n + alpha + 1. - rs) * old1 - (n + alpha) * old2 ) / (n + 1.)
+                new = ( (2*n + alpha + 1. - r) * old1 - (n + alpha) * old2 ) / (n + 1.)
             return new
     
     def __GradLaguerreP(self,r,N,k,alpha=None):
@@ -1311,7 +1276,6 @@ class Poly1D(object):
                 return self.__GradChebyshevP(r, N, k, 0)
             else:
                 r = np.array(r)
-                dP = np.mat(np.zeros((r.shape[0]))).T
                 if (N >= k):
                     dP = gammaF(alpha+beta+N+1.+k)/(2.**k * gammaF(alpha+beta+N+1.)) * np.sqrt(self.__GammaJacobiP(N-k,alpha+k,beta+k)/self.__GammaJacobiP(N,alpha,beta)) * self.__JacobiP(r,N-k,alpha+k,beta+k)
                 return dP
@@ -1558,7 +1522,25 @@ class Poly1D(object):
             return self.__GammaORTHPOL(N)
         else:
             print >> self.sdout, "[Spectral1D]: Gamma function not implemented yet for the selected polynomial %s" % self.poly
-        
+    
+    def __GradJacobiVandermonde(self,r,N,k,norm):
+        if (self.poly != JACOBI):
+            print >> self.sdout, "The method __GradJacobiP cannot be called with the actual type of polynomials. Actual type: '%s'" % self.poly
+        else:
+            # Unpack parameters
+            alpha,beta = self.params
+            
+            (al,be,ierr) = orthpol.drecur(N+1,6,alpha,beta)
+            dV = orthpol.vandermonde(r,N,al,be)
+            
+            if k > 0:
+                for n in range(N+1):
+                    if (n >= k):
+                        dV[:,n] = gammaF(alpha+beta+N+1.+k)/(2.**k * gammaF(alpha+beta+N+1.)) * np.sqrt(self.__GammaJacobiP(N-k,alpha+k,beta+k)/self.__GammaJacobiP(N,alpha,beta)) * dV[:,n]
+                    else:
+                        dV[:,n] = 0.
+            return dV
+    
     def GradVandermonde1D(self,r,N,k,norm=True):
         """
         GradVandermonde1D(): Initialize the ``k``-th gradient of the modal basis ``N`` at ``r``
@@ -1576,26 +1558,14 @@ class Poly1D(object):
             * ``V`` = (2d-array(``MxN``),float) Generalized Vandermonde matrix
         """
         
-        DVr = np.mat(np.zeros((r.shape[0],N+1)))
+        if (self.poly == JACOBI):
+            DVr = self.__GradJacobiVandermonde(r,N,k,norm)
+        else:
+            DVr = np.zeros((r.shape[0],N+1))
+            for i in range(0,N+1):
+                DVr[:,i] = self.GradEvaluate(r,i,k,norm)
         
-        for i in range(0,N+1):
-            DVr[:,i] = self.GradEvaluate(r,i,k,norm)
-        DVr = np.asarray(DVr)
-        
-#        # Normalization
-#        if (self.poly != ORTHPOL):
-#            if (self.poly == JACOBI):
-#                if (not norm):
-#                    # return non orthonormal basis
-#                    for i in range(0,N+1):
-#                        DVr[:,i] = DVr[:,i] * np.sqrt(self.__GammaJacobiP(i))
-#            else:
-#                if norm:
-#                    # return orthornormal basis
-#                    for i in range(0,N+1):
-#                        DVr[:,i] = DVr[:,i] / np.sqrt(self.Gamma(i))
-        
-        return np.asarray(DVr)
+        return DVr
         
     def AssemblyDerivativeMatrix(self, x, N, k):
         """
@@ -1740,7 +1710,7 @@ class Poly1D(object):
         
         if (self.poly == JACOBI) and ( AlmostEqual(alpha,-0.5) and AlmostEqual(beta,-0.5) ):
             # Chebyshev case
-            rr,w = self.JacobiCGL(N)
+            rr,w = self.__JacobiCGL(N)
             equal = True
             for i in range(0,N+1):
                 equal = AlmostEqual(r[i],rr[i])
@@ -1787,7 +1757,7 @@ class Poly1D(object):
         
         if (self.poly == JACOBI) and ( AlmostEqual(alpha,-0.5) and AlmostEqual(beta,-0.5) ):
             # Chebyshev case
-            rr,w = self.JacobiCGL(N)
+            rr,w = self.__JacobiCGL(N)
             equal = True
             for i in range(0,N+1):
                 equal = AlmostEqual(r[i],rr[i])
